package de.fh_zwickau.bumb_l_b.qos_tool;

import java.util.*;

import de.fh_zwickau.bumb_l_b.qos_tool.printer.StaticPrinterFactory;
import de.fh_zwickau.bumb_l_b.qos_tool.printer.StaticPrinterFactory.PrinterType;


/**
 * Launcher for the QoSTool application.
 * Christian Fiedler
 */
public class QoSToolLauncher {
	/**
	 * Starts the Quality of Service Tool
	 * @param args
	 */
	public static void main(String[] args) {
		String filename = "";
		PrinterType printerType = PrinterType.DEFAULT;
		
		for(String arg : args) {
			//TODO: Argumentbehandlung optimieren
			String[] pieces = arg.split(":", 2);
			
			switch(pieces[0]) {
				case "-o":
					if(pieces.length<2) { 
						System.out.println(Message.get("launcher.missing_output_type"));
						System.exit(1);
					}
					if("default".equals(pieces[1].toLowerCase())) { printerType = PrinterType.DEFAULT; }
					else if("bflow".equals(pieces[1].toLowerCase())) { printerType = PrinterType.BFLOW; }
					break;
					
				case "-f":
					if(pieces.length<2) { 
						System.out.println(Message.get("launcher.missing_filename"));
						System.exit(1);
					}
					filename = pieces[1];
					break;
					
//				case "-i":
//					if(pieces.length<2) { 
//						System.out.println("Please specify an input format for the -i option.");
//						System.exit(1);
//					}
//					if("epml".equals(pieces[1].toLowerCase())) { inputFormat = InputFormat.EPML; }
//					break;
					
				case "-h":
					printOptionsHelp();
					System.exit(0);
					break;
					
				default:
					System.out.println(String.format(Message.get("launcher.unknown_option"), pieces[0]));
					System.exit(1);
			}
		}
		
		QoSTool tool = new QoSTool(StaticPrinterFactory.getPrinterInstance(printerType));
		if(filename!="") { tool.loadEpcFromFile(filename); }
		tool.execute();		
	}
	
	public static void printOptionsHelp() {
		System.out.println(Message.get("launcher.help"));
	}
}
