package de.fh_zwickau.bumb_l_b.qos_tool.customJBPT;

import java.util.Set;

import org.jbpt.graph.abs.AbstractDirectedGraph;
import org.jbpt.graph.abs.AbstractMultiDirectedGraph;
import org.jbpt.pm.ControlFlow;
import org.jbpt.pm.IFlowNode;

import de.fh_zwickau.bumb_l_b.qos_tool.customJBPT.AttributeHelper.AttributeType;

public class CustomControlFlow<V extends IFlowNode> extends ControlFlow<V> implements IContainAttribute {
	private IContainAttribute attributes = new AttributeContainer();
	
	/**
	 * Create a new {@link ControlFlow} edge with the given parameter.
	 * @param graph the graph this edge should belong to
	 * @param from source node of this edge
	 * @param to target node of this edge
	 */
	public CustomControlFlow(AbstractMultiDirectedGraph<?, V> graph, V from, V to) {
		super(graph, from, to);
	}
	
	/**
	 * Create a new {@link ControlFlow} edge with the given parameter.
	 * @param graph the graph this edge should belong to
	 * @param from source node of this edge
	 * @param to target node of this edge
	 */
	public CustomControlFlow(AbstractDirectedGraph<?, V> graph, V from, V to) {
		super(graph, from, to);
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public ControlFlow<IFlowNode> clone() {
		CustomControlFlow<IFlowNode> clone = (CustomControlFlow<IFlowNode>) super.clone();
		for(AttributeType key : getAttributeKeys()) {
			clone.addAttribute(key, getAttribute(key));
		}
		return clone;
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public Set<AttributeType> getAttributeKeys() {
		return attributes.getAttributeKeys();
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void addAttribute(AttributeType type, Object value) {
		attributes.addAttribute(type, value);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void setAttribute(AttributeType type, Object value) {
		attributes.setAttribute(type, value);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public Object getAttribute(AttributeType type) {
		return attributes.getAttribute(type);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public boolean hasAttribute(AttributeType type) {
		return attributes.hasAttribute(type);
	}
}
