package de.fh_zwickau.bumb_l_b.qos_tool.loader;

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.jbpt.pm.ControlFlow;
import org.jbpt.pm.FlowNode;
import org.jbpt.pm.NonFlowNode;
import org.jbpt.pm.epc.IEpc;
import org.w3c.dom.Document;
import org.xml.sax.SAXException;

import de.fh_zwickau.bumb_l_b.qos_tool.Message;

public class EpmlEpcLoader implements IEpcLoader {
	private DocumentBuilderFactory dbFactory;
	private DocumentBuilder dBuilder;

	/**
	 * Creates a new Loader object
	 * @throws EpcLoaderException
	 */
	public EpmlEpcLoader() throws EpcLoaderException {
		dbFactory = DocumentBuilderFactory.newInstance();
		try {
			dBuilder = dbFactory.newDocumentBuilder();
		} catch (ParserConfigurationException e) {
			throw new EpcLoaderException(String.format(Message.get("loader.parser_initialisation_failed"), e.getLocalizedMessage()));
		}
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public IEpc<ControlFlow<FlowNode>, FlowNode, NonFlowNode> loadFile(File file) throws EpcLoaderException {
		try {
			Document doc = dBuilder.parse(file);
			return loadDocument(doc);
		} catch (SAXException e) {
			throw new EpcLoaderException(String.format(Message.get("loader.parser_file_not_parseable"), e.getLocalizedMessage()));
		} catch (IOException e) {
			throw new EpcLoaderException(String.format(Message.get("loader.parser_file_not_readable"), e.getLocalizedMessage()));
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public IEpc<ControlFlow<FlowNode>, FlowNode, NonFlowNode> loadFile(String filename) throws EpcLoaderException {
		File file = new File(filename);
		
		if(!file.exists() || !file.isFile() || !file.canRead()) {
			throw new EpcLoaderException(String.format(Message.get("loader.file_not_readable_or_not_exists"), filename));
		}
		
		return loadFile(file);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public IEpc<ControlFlow<FlowNode>, FlowNode, NonFlowNode> loadString(String epcString) throws EpcLoaderException {
		try {
			Document doc = dBuilder.parse(new ByteArrayInputStream(epcString.getBytes()));
			return loadDocument(doc);
		} catch (SAXException e) {
			throw new EpcLoaderException(String.format(Message.get("loader.parser_string_not_parseable"), e.getLocalizedMessage()));
		} catch (IOException e) {
			throw new EpcLoaderException(String.format(Message.get("loader.parser_string_not_readable"), e.getLocalizedMessage()));
		}
	}
	
	/**
	 * Parse the EPML DOM and return the first Model found.
	 * @param doc EPML DOM document
	 * @return first EPC model found
	 * @throws EpcLoaderException 
	 */
	private IEpc<ControlFlow<FlowNode>, FlowNode, NonFlowNode> loadDocument(Document doc) throws EpcLoaderException {
		IEpc<ControlFlow<FlowNode>, FlowNode, NonFlowNode> epc = null;
		
		EpmlParser parser = new EpmlParser(doc);
		try {
			epc = parser.getFirstModel();
		} catch (IllegalEpcAttributeFormatException e) {
			throw new EpcLoaderException(String.format(Message.get("loader.parser_error"), e.getMessage()), e.getElementReference());
		}
		
		return epc;
	}
}