package de.fh_zwickau.bumb_l_b.qos_tool.printer;

import java.io.PrintStream;

/**
 * Implements a simple default printer for human readable messages.
 * @author Christian Fiedler, Mathias Grunert
 */
public abstract class AbstractPrinter implements IPrinter {
	public enum MsgType {
		INFO,
		WARNING,
		ERROR
	}
	private PrintStream printStream = System.out;
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	public void printError(String msg) {
		printError(msg, "");
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void printWarning(String msg) {
		printWarning(msg, "");
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void printInfo(String msg) {
		printInfo(msg, "");
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void printError(String msg, String elementId) {
		printMessage(MsgType.ERROR, msg, elementId);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void printWarning(String msg, String elementId) {
		printMessage(MsgType.WARNING, msg, elementId);
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	public void printInfo(String msg, String elementId) {
		printMessage(MsgType.INFO, msg, elementId);
	}

	/**
	 * Sets a PrintStream for printer output
	 * @param printStream
	 */
	public void setOut(PrintStream printStream) {
		this.printStream = printStream;
	}
	
	/**
	 * Gets the currently set PrintStream for printer output. By default this is System.out
	 * @return
	 */
	protected PrintStream getPrintStream() {
		return printStream;
	}
	
	/**
	 * Print a message of the given Type related to an element.
	 * @param type
	 * @param message
	 * @param elementId
	 */
	protected abstract void printMessage(MsgType type, String message, String elementId);
}
