package de.fh_zwickau.bumb_l_b.qos_tool.processor;

import org.jbpt.pm.FlowNode;

import de.fh_zwickau.bumb_l_b.qos_tool.customJBPT.IContainAttribute;
import de.fh_zwickau.bumb_l_b.qos_tool.processor.pattern.PatternMatcher.PatternMatcherResult;

/**
 * Interface that defines Methods for pattern replacement strategies
 */
public interface IReplacementStrategy {
	/**
	 * Callback method for replacing the empty pattern
	 * @param result matching region data
	 * @return a single node containing summarized attribute data
	 * @throws ReplacementException
	 */
	public FlowNode replaceEmpty(final PatternMatcherResult result) throws ReplacementException;
	
	/**
	 * Callback method for replacing the linear pattern
	 * @param result matching region data
	 * @return a single node containing summarized attribute data
	 * @throws ReplacementException
	 */
	public FlowNode replaceLinear(final PatternMatcherResult result) throws ReplacementException;
	
	/**
	 * Callback method for replacing the and pattern
	 * @param result matching region data
	 * @return a single node containing summarized attribute data
	 * @throws ReplacementException
	 */
	public FlowNode replaceAnd(final PatternMatcherResult result) throws ReplacementException;
	
	/**
	 * Callback method for replacing the xor pattern
	 * @param result matching region data
	 * @return a single node containing summarized attribute data
	 * @throws ReplacementException
	 */
	public FlowNode replaceXor(final PatternMatcherResult result) throws ReplacementException;
	
	/**
	 * Callback method for replacing the cycle pattern
	 * @param result matching region data
	 * @return a single node containing summarized attribute data
	 * @throws ReplacementException
	 */
	public FlowNode replaceCycle(final PatternMatcherResult result) throws ReplacementException;
	
	/**
	 * Callback method for adding missing attribute values. Usually the values should be neutral i.e. identity elements for calculations.
	 * @param node the node to complete
	 */
	public void addMissingAttributeValues(final IContainAttribute node);
	
}
