package de.fh_zwickau.bumb_l_b.qos_tool.processor.pattern;

import org.jbpt.pm.ControlFlow;
import org.jbpt.pm.FlowNode;
import org.jbpt.pm.NonFlowNode;
import org.jbpt.pm.epc.IEpc;

import de.fh_zwickau.bumb_l_b.qos_tool.customJBPT.CustomAndConnector;
import de.fh_zwickau.bumb_l_b.qos_tool.customJBPT.CustomEvent;
import de.fh_zwickau.bumb_l_b.qos_tool.customJBPT.CustomFunction;

/**
 * A pattern matcher to find simple and constructs within an epc.
 */
public class AndPatternMatcher extends PatternMatcher {
	/**
	 * {@inheritDoc}
	 * @param epc
	 */
	public AndPatternMatcher(IEpc<ControlFlow<FlowNode>, FlowNode, NonFlowNode> epc) {
		super(epc);
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void findEntryPoints() {
		//Find entry points where and regions could start
		for(FlowNode gwNode : epc.getGateways()) {
			//only and splits
			if(gwNode instanceof CustomAndConnector && epc.getDirectSuccessors(gwNode).size() >= 2 && epc.getDirectPredecessors(gwNode).size() == 1) {
				results.add(new PatternMatcherResult(this, epc.getFirstDirectPredecessor(gwNode), gwNode));
			}
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected boolean isMatchingResult(PatternMatcherResult result) {
		result.nodes.add(result.getEntry());
		
		//all outgoing control flows
		for(FlowNode node : epc.getDirectSuccessors(result.getEntry())) {
			result.nodes.add(node);
			//first step should reach a function or event
			if(!(node instanceof CustomFunction || node instanceof CustomEvent)) {
				return false; //it doesn't match if it's neither a functor nor an event
			}

			//second step should reach the corresponding and join, it must be the same for every path 
			node = epc.getFirstDirectSuccessor(node);
			result.nodes.add(node);
			if(!(node instanceof CustomAndConnector)) {
				return false; //it doesn't match
			}
			if(epc.getDirectSuccessors(node).size() != 1) { //join must have exactly one outgoing control flow
				return false; //it's no valid join
			}
			
			//it is an and join
			if(result.getExit() == null) {
				result.exit = node; //register exit
				result.postExit = epc.getFirstDirectSuccessor(node);
			}
			else {
				if (node != result.getExit()) {
					return false; //it doesn't match if node and registered exit is not the same
				}
			}
		}
		
		return true;
	}
	
}
