package de.fh_zwickau.bumb_l_b.qos_tool.processor.pattern;

import org.jbpt.pm.ControlFlow;
import org.jbpt.pm.FlowNode;
import org.jbpt.pm.NonFlowNode;
import org.jbpt.pm.epc.IEpc;

import de.fh_zwickau.bumb_l_b.qos_tool.customJBPT.CustomEvent;
import de.fh_zwickau.bumb_l_b.qos_tool.customJBPT.CustomFunction;
import de.fh_zwickau.bumb_l_b.qos_tool.customJBPT.CustomXorConnector;

/**
 * A pattern matcher to find simple cycle constructs within an epc.
 */
public class CyclePatternMatcher extends PatternMatcher {
	/**
	 * {@inheritDoc}
	 * @param epc
	 */
	public CyclePatternMatcher(IEpc<ControlFlow<FlowNode>, FlowNode, NonFlowNode> epc) {
		super(epc);
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void findEntryPoints() {
		//Find entry points where and regions could start
		for(FlowNode gwNode : epc.getGateways()) {
			//only xor joins
			if(gwNode instanceof CustomXorConnector && epc.getDirectSuccessors(gwNode).size() == 1 && epc.getDirectPredecessors(gwNode).size() == 2) {
				results.add(new PatternMatcherResult(this, null, gwNode));
			}
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected boolean isMatchingResult(PatternMatcherResult result) {
		result.nodes.add(result.getEntry());
		
		//1) from the xor join walk two steps in the outgoing edge to find the corresponding xor split
		
		//must find a function or event one step ahead
		FlowNode suc1 = epc.getFirstDirectSuccessor(result.getEntry());
		if(!(suc1 instanceof CustomFunction || suc1 instanceof CustomEvent)) { return false; }
		result.nodes.add(suc1);
		
		//must find a xor gateway two steps ahead
		if(epc.getDirectSuccessors(suc1).size() != 1) { return false; }
		FlowNode suc2 = epc.getFirstDirectSuccessor(suc1);
		if(suc2 == null || !(suc2 instanceof CustomXorConnector)) { return false; }
		result.nodes.add(suc2);
		
		//2) split found...from there walk two steps along each outgoing edge
		//   exactly one path should reach the xor join where we started
		
		//the found xor gateway must have exactly two predecessors!
		if(epc.getDirectPredecessors(result.getEntry()).size() != 2) { return false; }
		//from that xor gateway walk all paths for two steps
		for(FlowNode pre : epc.getDirectPredecessors(result.getEntry())) {
			
			FlowNode pre2 = epc.getFirstDirectPredecessor(pre);
			if(pre == null) { return false; }
			else if(pre2 == suc2) { //exit (split) found!
				//assure that the first step was a function or event
				if(!(pre instanceof CustomFunction || pre instanceof CustomEvent)) { return false; }
				result.nodes.add(pre);
				result.exit = pre2;
				
				//find postExit node (the non cycling edge)
				for(FlowNode node : epc.getDirectSuccessors(result.exit)) {
					if(node != pre) { result.postExit = node; }
				}
			}
			else { //preEntry found (on the first step)!
				result.preEntry = pre;
			}
			
		}
		
		//if result is incomplete, it is no match
		if(result.exit == null) { return false; }
		
		return true;
	}
	
}
