package de.fh_zwickau.bumb_l_b.qos_tool.processor.pattern;

import org.jbpt.pm.ControlFlow;
import org.jbpt.pm.FlowNode;
import org.jbpt.pm.Gateway;
import org.jbpt.pm.NonFlowNode;
import org.jbpt.pm.epc.IEpc;

/**
 * A pattern matcher to find empty epc regions between gateways.
 */
public class EmptyPatternMatcher extends PatternMatcher {
	/**
	 * {@inheritDoc}
	 * @param epc
	 */
	public EmptyPatternMatcher(IEpc<ControlFlow<FlowNode>, FlowNode, NonFlowNode> epc) {
		super(epc);
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void findEntryPoints() {
		PatternMatcherResult result;
		
		//Find entry points where empties could start
		//epc entry points is a gateway
		for(FlowNode node : epc.getEntries()) {
			if(node instanceof Gateway) {
				result = new PatternMatcherResult(this, null, null, null, node);
				result.nodes.add(node);
				results.add(result);
			}
		}
		//next nodes after gateways
		for(FlowNode gwNode : epc.getGateways()) {
			if(epc.getDirectSuccessors(gwNode).size() == 0) {
				//last element in epc is a gateway
				result = new PatternMatcherResult(this, gwNode, null, null, null);
				result.nodes.add(gwNode);
				results.add(result);
			}
			else {
				for(FlowNode node : epc.getDirectSuccessors(gwNode)) {
					//gateway directly followed by a gateway
					if(node instanceof Gateway) {
						result = new PatternMatcherResult(this, gwNode, null, null, node);
						result.nodes.add(gwNode);
						result.nodes.add(node);
						results.add(result);
					}
				}
			}
		}
	}

	/**
	 * Every result generated by {@link #findEntryPoints()} matches.
	 * {@inheritDoc}
	 */
	@Override
	protected boolean isMatchingResult(PatternMatcherResult result) {
		return true;
	}

	
}
