package de.fh_zwickau.bumb_l_b.qos_tool.processor.pattern;

import org.jbpt.pm.ControlFlow;
import org.jbpt.pm.FlowNode;
import org.jbpt.pm.NonFlowNode;
import org.jbpt.pm.epc.IEpc;

import de.fh_zwickau.bumb_l_b.qos_tool.customJBPT.CustomEvent;
import de.fh_zwickau.bumb_l_b.qos_tool.customJBPT.CustomFunction;
import de.fh_zwickau.bumb_l_b.qos_tool.customJBPT.IContainAttribute;

/**
 * A pattern matcher to find linear epc regions. The matching regions consists only of function and event nodes.
 */
public class LinearPatternMatcher extends PatternMatcher {
	/**
	 * {@inheritDoc}
	 * @param epc
	 */
	public LinearPatternMatcher(IEpc<ControlFlow<FlowNode>, FlowNode, NonFlowNode> epc) {
		super(epc);
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void findEntryPoints() {
		//Find entry points where linears could start
		//epc entry points
		for(FlowNode node : epc.getEntries()) {
			if(node instanceof CustomFunction || node instanceof CustomEvent) {
				results.add(new PatternMatcherResult(this, null, node));
			}
		}
		//next nodes after gateways
		for(FlowNode gwNode : epc.getGateways()) {
			for(FlowNode node : epc.getDirectSuccessors(gwNode)) {
				//add only functions and events
				if(node instanceof CustomFunction || node instanceof CustomEvent) {
					results.add(new PatternMatcherResult(this, gwNode, node));
				}
			}
		}
	}

	/**
	 * {@inheritDoc}
	 */
	@Override
	protected boolean isMatchingResult(PatternMatcherResult result) {
		//follow chain until next gateway occurs or chain ends
		FlowNode node = result.getEntry();
		while(node != null) {
			if(node instanceof CustomFunction || node instanceof CustomEvent) {
				result.nodes.add(node);
				result.exit = node;
			}
			else { //gateway found
				result.postExit = node;
				break;
			}				
			node = epc.getFirstDirectSuccessor(node);				
		}
		
		//linear matches only if more than one node matches
		if(result.nodes.size() <= 1) {
			return false;
		};
		
		return true;
	}

	
}
