package de.fh_zwickau.bumb_l_b.qos_tool.processor.pattern;

import org.jbpt.pm.ControlFlow;
import org.jbpt.pm.FlowNode;
import org.jbpt.pm.NonFlowNode;
import org.jbpt.pm.epc.IEpc;

/**
 * A pattern matcher to find useless gateways.
 */
public class UselessGatewayPatternMatcher extends PatternMatcher {
	/**
	 * {@inheritDoc}
	 * @param epc
	 */
	public UselessGatewayPatternMatcher(IEpc<ControlFlow<FlowNode>, FlowNode, NonFlowNode> epc) {
		super(epc);
	}
	
	/**
	 * {@inheritDoc}
	 */
	@Override
	protected void findEntryPoints() {
		PatternMatcherResult result;
		
		//gateways with max 1 outgoing and 1 incoming control flow
		for(FlowNode gwNode : epc.getGateways()) {
			if(epc.getDirectSuccessors(gwNode).size() <= 1 && epc.getDirectPredecessors(gwNode).size() <= 1) {
				result = new PatternMatcherResult(this, epc.getFirstDirectPredecessor(gwNode), gwNode, gwNode, epc.getFirstDirectSuccessor(gwNode));
				result.nodes.add(gwNode);
				results.add(result);
			}
		}
	}

	/**
	 * Every result generated by {@link #findEntryPoints()} matches.
	 * {@inheritDoc}
	 */
	@Override
	protected boolean isMatchingResult(PatternMatcherResult result) {
		return true;
	}

	
}
