package de.fh_zwickau.bumb_l_b.qos_tool.loader;

import static org.junit.Assert.*;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;

import org.jbpt.pm.ControlFlow;
import org.jbpt.pm.FlowNode;
import org.jbpt.pm.NonFlowNode;
import org.jbpt.pm.epc.IEpc;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import de.fh_zwickau.bumb_l_b.qos_tool.customJBPT.AttributeHelper.AttributeType;
import de.fh_zwickau.bumb_l_b.qos_tool.customJBPT.CustomAndConnector;
import de.fh_zwickau.bumb_l_b.qos_tool.customJBPT.CustomEvent;
import de.fh_zwickau.bumb_l_b.qos_tool.customJBPT.CustomFunction;
import de.fh_zwickau.bumb_l_b.qos_tool.customJBPT.CustomOrConnector;
import de.fh_zwickau.bumb_l_b.qos_tool.customJBPT.CustomXorConnector;
import de.fh_zwickau.bumb_l_b.qos_tool.customJBPT.IContainAttribute;

public class EpmlEpcLoaderTest {
	private static final double ATTRIBUTE_DELTA = 1e-7;
	EpmlEpcLoader loader;
	
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
	}

	@Before
	public void setUp() throws Exception {
		loader = new EpmlEpcLoader();
	}

	
	@Test
	public void testLoadFileFile() throws EpcLoaderException {
		IEpc<ControlFlow<FlowNode>, FlowNode, NonFlowNode> epc = loader.loadFile(new File("testContent/Test.epml"));
		assertNotNull(epc);
	}
	
	@Test
	public void testLoadFileString() throws EpcLoaderException {
		IEpc<ControlFlow<FlowNode>, FlowNode, NonFlowNode> epc = loader.loadFile("testContent/Test.epml");
		assertNotNull(epc);
	}

	@Test
	public void testLoadString() throws EpcLoaderException, IOException, URISyntaxException {
//		try (BufferedReader br = new BufferedReader(new FileReader("testContent/Test.epml")))
		BufferedReader br = new BufferedReader(new FileReader("testContent/Test.epml"));
		String str = "";
		try {
			StringBuilder sb = new StringBuilder();
			String line = br.readLine();

			while (line != null) {
				sb.append(line);
				sb.append(System.lineSeparator());
				line = br.readLine();
			}
			str = sb.toString();
		} finally {
			br.close();
		}

		IEpc<ControlFlow<FlowNode>, FlowNode, NonFlowNode> epc = loader.loadString(str);
		assertNotNull(epc);
	}

	//detaillierte Tests für verschiedene Modelle (zu speichern in testContent)
	@Test
	public void testLoadEpc_Check_Start_End_Funtions_Events_BflowId_Cost_Time_Reliability() throws EpcLoaderException {
		IEpc<ControlFlow<FlowNode>, FlowNode, NonFlowNode> epc = loader.loadFile("testContent/test_tcr_linear_small_1.epml");
		assertNotNull(epc);
		
		ArrayList<FlowNode> nodes;
		nodes = new ArrayList<FlowNode>(epc.getEntries());
		assertSame(1, nodes.size());
		assertTrue("Incorrect type", nodes.get(0) instanceof CustomEvent);
		assertEquals("Startereignis", (nodes.get(0)).getName());
		assertEquals("_2Oe74NEZEeOs355zyZvogA", ((IContainAttribute)nodes.get(0)).getAttribute(AttributeType.BFLOW_ID));
		
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(1, nodes.size());
		assertTrue("Incorrect type", nodes.get(0) instanceof CustomFunction);
		assertEquals("F1", (nodes.get(0)).getName());
		assertEquals("_4HIfkNEZEeOs355zyZvogA", ((IContainAttribute)nodes.get(0)).getAttribute(AttributeType.BFLOW_ID));
		assertEquals(100.0, (Double) ((IContainAttribute)nodes.get(0)).getAttribute(AttributeType.COST), ATTRIBUTE_DELTA);
		assertEquals(0.9, (Double) ((IContainAttribute)nodes.get(0)).getAttribute(AttributeType.RELIABILITY), ATTRIBUTE_DELTA);
		assertEquals(10, (Double) ((IContainAttribute)nodes.get(0)).getAttribute(AttributeType.TIME), ATTRIBUTE_DELTA);
		
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(1, nodes.size());
		assertTrue("Incorrect type", nodes.get(0) instanceof CustomEvent);
		assertEquals("E1", (nodes.get(0)).getName());
		assertEquals("_7-i2ANEZEeOs355zyZvogA", ((IContainAttribute)nodes.get(0)).getAttribute(AttributeType.BFLOW_ID));
		
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(1, nodes.size());
		assertTrue("Incorrect type", nodes.get(0) instanceof CustomFunction);
		assertEquals("F2", (nodes.get(0)).getName());
		assertEquals("__su_ANEZEeOs355zyZvogA", ((IContainAttribute)nodes.get(0)).getAttribute(AttributeType.BFLOW_ID));
		assertEquals(200.0, (Double) ((IContainAttribute)nodes.get(0)).getAttribute(AttributeType.COST), ATTRIBUTE_DELTA);
		assertEquals(0.8, (Double) ((IContainAttribute)nodes.get(0)).getAttribute(AttributeType.RELIABILITY), ATTRIBUTE_DELTA);
		assertEquals(20, (Double) ((IContainAttribute)nodes.get(0)).getAttribute(AttributeType.TIME), ATTRIBUTE_DELTA);
		
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(1, nodes.size());
		assertTrue("Incorrect type", nodes.get(0) instanceof CustomEvent);
		assertEquals("Endereignis", (nodes.get(0)).getName());
		assertEquals("_BJt6YNEaEeOs355zyZvogA", ((IContainAttribute)nodes.get(0)).getAttribute(AttributeType.BFLOW_ID));
		
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(0, nodes.size());
	}
	
	@Test
	public void testLoadEpc_Check_AND__with_n_out_of_m() throws EpcLoaderException {
		IEpc<ControlFlow<FlowNode>, FlowNode, NonFlowNode> epc = loader.loadFile("testContent/test_tcrn_and_small_1.epml");
		assertNotNull(epc);
		
		ArrayList<FlowNode> nodes, nodesTmp1, nodesTmp2;
		nodes = new ArrayList<FlowNode>(epc.getEntries());
		assertSame(1, nodes.size());
		assertTrue("Incorrect type", nodes.get(0) instanceof CustomEvent);
		assertEquals("Startereignis", (nodes.get(0)).getName());
		assertEquals("_2Oe74NEZEeOs355zyZvogA", ((IContainAttribute)nodes.get(0)).getAttribute(AttributeType.BFLOW_ID));
		
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(1, nodes.size());
		assertTrue("Incorrect type", nodes.get(0) instanceof CustomAndConnector);
		assertEquals("_TFzEcNElEeOrKdkTrDz40Q", ((IContainAttribute)nodes.get(0)).getAttribute(AttributeType.BFLOW_ID));
		assertNull(((IContainAttribute)nodes.get(0)).getAttribute(AttributeType.N_OUT_OF_M));
		
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(2, nodes.size());
		//Nach bflow id sortieren, damit Reihenfolge klar ist
		Collections.sort(nodes, new Comparator<FlowNode>() {
			@Override
			public int compare(FlowNode n1, FlowNode n2) {
				return n1.getId().compareTo(n2.getId());
			}
		});
		assertTrue("Incorrect type", nodes.get(0) instanceof CustomFunction);
		assertEquals("F1", (nodes.get(0)).getName());
		assertEquals("_4HIfkNEZEeOs355zyZvogA", ((IContainAttribute)nodes.get(0)).getAttribute(AttributeType.BFLOW_ID));
		assertEquals(100.0, (Double) ((IContainAttribute)nodes.get(0)).getAttribute(AttributeType.COST), ATTRIBUTE_DELTA);
		assertEquals(0.9, (Double) ((IContainAttribute)nodes.get(0)).getAttribute(AttributeType.RELIABILITY), ATTRIBUTE_DELTA);
		assertEquals(10.0, (Double) ((IContainAttribute)nodes.get(0)).getAttribute(AttributeType.TIME), ATTRIBUTE_DELTA);
		//und
		assertTrue("Incorrect type", nodes.get(1) instanceof CustomFunction);
		assertEquals("F2", (nodes.get(1)).getName());
		assertEquals("__su_ANEZEeOs355zyZvogA", ((IContainAttribute)nodes.get(1)).getAttribute(AttributeType.BFLOW_ID));
		assertEquals(200.0, (Double) ((IContainAttribute)nodes.get(1)).getAttribute(AttributeType.COST), ATTRIBUTE_DELTA);
		assertEquals(0.8, (Double) ((IContainAttribute)nodes.get(1)).getAttribute(AttributeType.RELIABILITY), ATTRIBUTE_DELTA);
		assertEquals(20.0, (Double) ((IContainAttribute)nodes.get(1)).getAttribute(AttributeType.TIME), ATTRIBUTE_DELTA);
		
		nodesTmp1 = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(1, nodesTmp1.size());
		nodesTmp2 = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(1)));
		assertSame(1, nodesTmp2.size());
		assertSame(nodesTmp1.get(0), nodesTmp2.get(0));
		nodes = nodesTmp1;
		assertTrue("Incorrect type", nodes.get(0) instanceof CustomAndConnector);
		assertEquals("_Tl45wNElEeOrKdkTrDz40Q", ((IContainAttribute)nodes.get(0)).getAttribute(AttributeType.BFLOW_ID));
		assertEquals(2, ((IContainAttribute)nodes.get(0)).getAttribute(AttributeType.N_OUT_OF_M));
		
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(1, nodes.size());
		assertTrue("Incorrect type", nodes.get(0) instanceof CustomEvent);
		assertEquals("Endereignis", (nodes.get(0)).getName());
		assertEquals("_BJt6YNEaEeOs355zyZvogA", ((IContainAttribute)nodes.get(0)).getAttribute(AttributeType.BFLOW_ID));
		
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(0, nodes.size());
	}
	
	@Test
	public void testLoadEpc_Check_OR() throws EpcLoaderException {
		IEpc<ControlFlow<FlowNode>, FlowNode, NonFlowNode> epc = loader.loadFile("testContent/test_tcr_or_small_1.epml");
		assertNotNull(epc);
		
		ArrayList<FlowNode> nodes, nodesTmp1, nodesTmp2;
		FlowNode node;
		
		nodes = new ArrayList<FlowNode>(epc.getEntries());
		assertSame(1, nodes.size());
		node = nodes.get(0);
		assertTrue("Incorrect type", node instanceof CustomEvent);
		assertEquals("Startereignis", node.getName());
		assertEquals("_2Oe74NEZEeOs355zyZvogA", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(1, nodes.size());
		node = nodes.get(0);
		assertTrue("Incorrect type", node instanceof CustomFunction);
		assertEquals("F0", (node).getName());
		assertEquals("_EXRy0NErEeOrKdkTrDz40Q", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		assertEquals(0.0, (Double) ((IContainAttribute)nodes.get(0)).getAttribute(AttributeType.COST), ATTRIBUTE_DELTA);
		assertEquals(1.0, (Double) ((IContainAttribute)nodes.get(0)).getAttribute(AttributeType.RELIABILITY), ATTRIBUTE_DELTA);
		assertEquals(0.0, (Double) ((IContainAttribute)nodes.get(0)).getAttribute(AttributeType.TIME), ATTRIBUTE_DELTA);
		
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(1, nodes.size());
		node = nodes.get(0);
		assertTrue("Incorrect type", node instanceof CustomOrConnector);
		assertEquals("_GdXVkNErEeOrKdkTrDz40Q", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(2, nodes.size());
		//Nach bflow id sortieren, damit Reihenfolge klar ist
		Collections.sort(nodes, new Comparator<FlowNode>() {
			@Override
			public int compare(FlowNode n1, FlowNode n2) {
				return n1.getId().compareTo(n2.getId());
			}
		});
		
		node = nodes.get(0);
		assertTrue("Incorrect type", node instanceof CustomEvent);
		assertEquals("E1", (node).getName());
		assertEquals("_KOxxsNErEeOrKdkTrDz40Q", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		//oder
		node = nodes.get(1);
		assertTrue("Incorrect type", node instanceof CustomEvent);
		assertEquals("E2", node.getName());
		assertEquals("_LbcZINErEeOrKdkTrDz40Q", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		
		nodesTmp1 = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(1, nodesTmp1.size());
		node = nodesTmp1.get(0);
		assertTrue("Incorrect type", node instanceof CustomFunction);
		assertEquals("F1", node.getName());
		assertEquals("_4HIfkNEZEeOs355zyZvogA", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		assertEquals(100.0, (Double) ((IContainAttribute)node).getAttribute(AttributeType.COST), ATTRIBUTE_DELTA);
		assertEquals(0.9, (Double) ((IContainAttribute)node).getAttribute(AttributeType.RELIABILITY), ATTRIBUTE_DELTA);
		assertEquals(10, (Double) ((IContainAttribute)node).getAttribute(AttributeType.TIME), ATTRIBUTE_DELTA);
		//oder
		nodesTmp2 = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(1)));
		assertSame(1, nodesTmp2.size());
		node = nodesTmp2.get(0);
		assertTrue("Incorrect type", node instanceof CustomFunction);
		assertEquals("F2", node.getName());
		assertEquals("__su_ANEZEeOs355zyZvogA", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		assertEquals(200.0, (Double) ((IContainAttribute)node).getAttribute(AttributeType.COST), ATTRIBUTE_DELTA);
		assertEquals(0.8, (Double) ((IContainAttribute)node).getAttribute(AttributeType.RELIABILITY), ATTRIBUTE_DELTA);
		assertEquals(20.0, (Double) ((IContainAttribute)node).getAttribute(AttributeType.TIME), ATTRIBUTE_DELTA);
		
		nodesTmp1 = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodesTmp1.get(0)));
		assertSame(1, nodesTmp1.size());
		nodesTmp2 = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodesTmp2.get(0)));
		assertSame(1, nodesTmp2.size());
		assertSame(nodesTmp1.get(0), nodesTmp2.get(0));
		nodes = nodesTmp1;
		node = nodes.get(0);
		assertTrue("Incorrect type", node instanceof CustomOrConnector);
		assertEquals("_uGAwMNEqEeOrKdkTrDz40Q", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(1, nodes.size());
		node = nodes.get(0);
		assertTrue("Incorrect type", node instanceof CustomEvent);
		assertEquals("Endereignis", node.getName());
		assertEquals("_BJt6YNEaEeOs355zyZvogA", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(node));
		assertSame(0, nodes.size());
	}
	
	@Test
	public void testLoadEpc_Check_XOR() throws EpcLoaderException {
		IEpc<ControlFlow<FlowNode>, FlowNode, NonFlowNode> epc = loader.loadFile("testContent/test_tcr_xor_small_1.epml");
		assertNotNull(epc);
		
		ArrayList<FlowNode> nodes, nodesTmp1, nodesTmp2;
		FlowNode node;
		
		nodes = new ArrayList<FlowNode>(epc.getEntries());
		assertSame(1, nodes.size());
		node = nodes.get(0);
		assertTrue("Incorrect type", node instanceof CustomEvent);
		assertEquals("Startereignis", node.getName());
		assertEquals("_2Oe74NEZEeOs355zyZvogA", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(1, nodes.size());
		node = nodes.get(0);
		assertTrue("Incorrect type", node instanceof CustomFunction);
		assertEquals("F0", (node).getName());
		assertEquals("_EXRy0NErEeOrKdkTrDz40Q", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		assertEquals(0.0, (Double) ((IContainAttribute)node).getAttribute(AttributeType.COST), ATTRIBUTE_DELTA);
		assertEquals(1.0, (Double) ((IContainAttribute)node).getAttribute(AttributeType.RELIABILITY), ATTRIBUTE_DELTA);
		assertEquals(0.0, (Double) ((IContainAttribute)node).getAttribute(AttributeType.TIME), ATTRIBUTE_DELTA);
		
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(1, nodes.size());
		node = nodes.get(0);
		assertTrue("Incorrect type", node instanceof CustomXorConnector);
		assertEquals("_sm_3sNEuEeOrKdkTrDz40Q", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		
//		nodeTmp = node; //letzten Knoten merken
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(2, nodes.size());
		//Nach bflow id sortieren, damit Reihenfolge klar ist
		Collections.sort(nodes, new Comparator<FlowNode>() {
			@Override
			public int compare(FlowNode n1, FlowNode n2) {
				return n1.getId().compareTo(n2.getId());
			}
		});
		
		//Wahrscheinlichkeiten an Kanten prüfen
		assertEquals(0.2, (Double) ((IContainAttribute)epc.getEdge(node, nodes.get(0))).getAttribute(AttributeType.PROBABILITY), ATTRIBUTE_DELTA);
		assertEquals(0.8, (Double) ((IContainAttribute)epc.getEdge(node, nodes.get(1))).getAttribute(AttributeType.PROBABILITY), ATTRIBUTE_DELTA);
		
		node = nodes.get(0);
		assertTrue("Incorrect type", node instanceof CustomEvent);
		assertEquals("E1", (node).getName());
		assertEquals("_KOxxsNErEeOrKdkTrDz40Q", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		//oder
		node = nodes.get(1);
		assertTrue("Incorrect type", node instanceof CustomEvent);
		assertEquals("E2", node.getName());
		assertEquals("_LbcZINErEeOrKdkTrDz40Q", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		
		nodesTmp1 = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(1, nodesTmp1.size());
		node = nodesTmp1.get(0);
		assertTrue("Incorrect type", node instanceof CustomFunction);
		assertEquals("F1", node.getName());
		assertEquals("_4HIfkNEZEeOs355zyZvogA", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		assertEquals(100.0, (Double) ((IContainAttribute)node).getAttribute(AttributeType.COST), ATTRIBUTE_DELTA);
		assertEquals(0.9, (Double) ((IContainAttribute)node).getAttribute(AttributeType.RELIABILITY), ATTRIBUTE_DELTA);
		assertEquals(10.0, (Double) ((IContainAttribute)node).getAttribute(AttributeType.TIME), ATTRIBUTE_DELTA);
		//oder
		nodesTmp2 = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(1)));
		assertSame(1, nodesTmp2.size());
		node = nodesTmp2.get(0);
		assertTrue("Incorrect type", node instanceof CustomFunction);
		assertEquals("F2", node.getName());
		assertEquals("__su_ANEZEeOs355zyZvogA", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		assertEquals(200.0, (Double) ((IContainAttribute)node).getAttribute(AttributeType.COST), ATTRIBUTE_DELTA);
		assertEquals(0.8, (Double) ((IContainAttribute)node).getAttribute(AttributeType.RELIABILITY), ATTRIBUTE_DELTA);
		assertEquals(20.0, (Double) ((IContainAttribute)node).getAttribute(AttributeType.TIME), ATTRIBUTE_DELTA);
		
		nodesTmp1 = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodesTmp1.get(0)));
		assertSame(1, nodesTmp1.size());
		nodesTmp2 = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodesTmp2.get(0)));
		assertSame(1, nodesTmp2.size());
		assertSame(nodesTmp1.get(0), nodesTmp2.get(0));
		nodes = nodesTmp1;
		node = nodes.get(0);
		assertTrue("Incorrect type", node instanceof CustomXorConnector);
		assertEquals("_s8DhsNEuEeOrKdkTrDz40Q", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(1, nodes.size());
		node = nodes.get(0);
		assertTrue("Incorrect type", node instanceof CustomEvent);
		assertEquals("Endereignis", node.getName());
		assertEquals("_BJt6YNEaEeOs355zyZvogA", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(node));
		assertSame(0, nodes.size());
	}
	
	@Test
	public void testLoadEpc_Check_Loop_Geometric() throws EpcLoaderException {
		IEpc<ControlFlow<FlowNode>, FlowNode, NonFlowNode> epc = loader.loadFile("testContent/test_tcr_loop_geometric_small_1.epml");
		assertNotNull(epc);
		
		ArrayList<FlowNode> nodes, nodesTmp1, nodesTmp2;
		FlowNode node;
		
		nodes = new ArrayList<FlowNode>(epc.getEntries());
		assertSame(1, nodes.size());
		node = nodes.get(0);
		assertTrue("Incorrect type", node instanceof CustomEvent);
		assertEquals("Startereignis", node.getName());
		assertEquals("_2Oe74NEZEeOs355zyZvogA", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(1, nodes.size());
		node = nodes.get(0);
		assertTrue("Incorrect type", node instanceof CustomFunction);
		assertEquals("F0", (node).getName());
		assertEquals("_EXRy0NErEeOrKdkTrDz40Q", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		assertEquals(0.0, (Double) ((IContainAttribute)node).getAttribute(AttributeType.COST), ATTRIBUTE_DELTA);
		assertEquals(1.0, (Double) ((IContainAttribute)node).getAttribute(AttributeType.RELIABILITY), ATTRIBUTE_DELTA);
		assertEquals(0.0, (Double) ((IContainAttribute)node).getAttribute(AttributeType.TIME), ATTRIBUTE_DELTA);
		
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(1, nodes.size());
		FlowNode xorJoinNode = nodes.get(0);
		node = nodes.get(0);
		assertTrue("Incorrect type", node instanceof CustomXorConnector);
		assertEquals("_sm_3sNEuEeOrKdkTrDz40Q", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		
//		nodeTmp = node; //letzten Knoten merken
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(1, nodes.size());
		//Nach bflow id sortieren, damit Reihenfolge klar ist
		Collections.sort(nodes, new Comparator<FlowNode>() {
			@Override
			public int compare(FlowNode n1, FlowNode n2) {
				return n1.getId().compareTo(n2.getId());
			}
		});
				
		node = nodes.get(0);
		assertTrue("Incorrect type", node instanceof CustomEvent);
		assertEquals("E1", (node).getName());
		assertEquals("_KOxxsNErEeOrKdkTrDz40Q", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		
		nodesTmp1 = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(1, nodesTmp1.size());
		node = nodesTmp1.get(0);
		assertTrue("Incorrect type", node instanceof CustomFunction);
		assertEquals("F1", node.getName());
		assertEquals("_4HIfkNEZEeOs355zyZvogA", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		assertEquals(100.0, (Double) ((IContainAttribute)node).getAttribute(AttributeType.COST), ATTRIBUTE_DELTA);
		assertEquals(0.9, (Double) ((IContainAttribute)node).getAttribute(AttributeType.RELIABILITY), ATTRIBUTE_DELTA);
		assertEquals(10.0, (Double) ((IContainAttribute)node).getAttribute(AttributeType.TIME), ATTRIBUTE_DELTA);

		nodesTmp1 = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodesTmp1.get(0)));
		assertSame(1, nodesTmp1.size());
		nodes = nodesTmp1;
		node = nodes.get(0);
		assertTrue("Incorrect type", node instanceof CustomXorConnector);
		assertEquals("_s8DhsNEuEeOrKdkTrDz40Q", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		
		
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(2, nodes.size());
		//Nach bflow id sortieren, damit Reihenfolge klar ist
		Collections.sort(nodes, new Comparator<FlowNode>() {
			@Override
			public int compare(FlowNode n1, FlowNode n2) {
				return n1.getId().compareTo(n2.getId());
			}
		});

		//Wahrscheinlichkeiten an Kanten prüfen
		assertEquals(0.7, (Double) ((IContainAttribute)epc.getEdge(node, nodes.get(0))).getAttribute(AttributeType.PROBABILITY), ATTRIBUTE_DELTA);
		assertEquals(0.3, (Double) ((IContainAttribute)epc.getEdge(node, nodes.get(1))).getAttribute(AttributeType.PROBABILITY), ATTRIBUTE_DELTA);

		//Endereignis
		node = nodes.get(0);
		assertTrue("Incorrect type", node instanceof CustomEvent);
		assertEquals("Endereignis", node.getName());
		assertEquals("_BJt6YNEaEeOs355zyZvogA", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		//und da ist die Kette zuende
		nodesTmp1 = new ArrayList<FlowNode>(epc.getDirectSuccessors(node));
		assertSame(0, nodesTmp1.size());

//		//oder Schleife:
		node = nodes.get(1);
		assertTrue("Incorrect type", node instanceof CustomEvent);
		assertEquals("E2", node.getName());
		assertEquals("_LbcZINErEeOrKdkTrDz40Q", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));

		nodesTmp2 = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(1)));
		assertSame(1, nodesTmp2.size());
		node = nodesTmp2.get(0);
		assertTrue("Incorrect type", node instanceof CustomFunction);
		assertEquals("F2", node.getName());
		assertEquals("__su_ANEZEeOs355zyZvogA", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		assertEquals(200.0, (Double) ((IContainAttribute)node).getAttribute(AttributeType.COST), ATTRIBUTE_DELTA);
		assertEquals(0.8, (Double) ((IContainAttribute)node).getAttribute(AttributeType.RELIABILITY), ATTRIBUTE_DELTA);
		assertEquals(20.0, (Double) ((IContainAttribute)node).getAttribute(AttributeType.TIME), ATTRIBUTE_DELTA);
	}
	
	@Test
	public void testLoadEpc_Check_Loop_Tuple() throws EpcLoaderException {
		IEpc<ControlFlow<FlowNode>, FlowNode, NonFlowNode> epc = loader.loadFile("testContent/test_tcr_loop_tuple_small_1.epml");
		assertNotNull(epc);
		
		ArrayList<FlowNode> nodes, nodesTmp1, nodesTmp2;
		FlowNode node;
		
		nodes = new ArrayList<FlowNode>(epc.getEntries());
		assertSame(1, nodes.size());
		node = nodes.get(0);
		assertTrue("Incorrect type", node instanceof CustomEvent);
		assertEquals("Startereignis", node.getName());
		assertEquals("_2Oe74NEZEeOs355zyZvogA", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(1, nodes.size());
		node = nodes.get(0);
		assertTrue("Incorrect type", node instanceof CustomFunction);
		assertEquals("F0", (node).getName());
		assertEquals("_EXRy0NErEeOrKdkTrDz40Q", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		assertEquals(0.0, (Double) ((IContainAttribute)node).getAttribute(AttributeType.COST), ATTRIBUTE_DELTA);
		assertEquals(1.0, (Double) ((IContainAttribute)node).getAttribute(AttributeType.RELIABILITY), ATTRIBUTE_DELTA);
		assertEquals(0.0, (Double) ((IContainAttribute)node).getAttribute(AttributeType.TIME), ATTRIBUTE_DELTA);
		
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(1, nodes.size());
		FlowNode xorJoinNode = nodes.get(0);
		node = nodes.get(0);
		assertTrue("Incorrect type", node instanceof CustomXorConnector);
		assertEquals("_sm_3sNEuEeOrKdkTrDz40Q", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		
//		nodeTmp = node; //letzten Knoten merken
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(1, nodes.size());
		//Nach bflow id sortieren, damit Reihenfolge klar ist
		Collections.sort(nodes, new Comparator<FlowNode>() {
			@Override
			public int compare(FlowNode n1, FlowNode n2) {
				return n1.getId().compareTo(n2.getId());
			}
		});
		
		node = nodes.get(0);
		assertTrue("Incorrect type", node instanceof CustomEvent);
		assertEquals("E1", (node).getName());
		assertEquals("_KOxxsNErEeOrKdkTrDz40Q", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		
		nodesTmp1 = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(1, nodesTmp1.size());
		node = nodesTmp1.get(0);
		assertTrue("Incorrect type", node instanceof CustomFunction);
		assertEquals("F1", node.getName());
		assertEquals("_4HIfkNEZEeOs355zyZvogA", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		assertEquals(100.0, (Double) ((IContainAttribute)node).getAttribute(AttributeType.COST), ATTRIBUTE_DELTA);
		assertEquals(0.9, (Double) ((IContainAttribute)node).getAttribute(AttributeType.RELIABILITY), ATTRIBUTE_DELTA);
		assertEquals(10.0, (Double) ((IContainAttribute)node).getAttribute(AttributeType.TIME), ATTRIBUTE_DELTA);
		
		nodesTmp1 = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodesTmp1.get(0)));
		assertSame(1, nodesTmp1.size());
		nodes = nodesTmp1;
		node = nodes.get(0);
		assertTrue("Incorrect type", node instanceof CustomXorConnector);
		assertEquals("_s8DhsNEuEeOrKdkTrDz40Q", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		
		
		nodes = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(0)));
		assertSame(2, nodes.size());
		//Nach bflow id sortieren, damit Reihenfolge klar ist
		Collections.sort(nodes, new Comparator<FlowNode>() {
			@Override
			public int compare(FlowNode n1, FlowNode n2) {
				return n1.getId().compareTo(n2.getId());
			}
		});
		
		//Wahrscheinlichkeiten an Kanten prüfen
		//bei der Schleife ist die Wahrscheinlichkeit als Tupel gegeben
		List<Double> tuple = (List<Double>)((IContainAttribute)epc.getEdge(node, nodes.get(1))).getAttribute(AttributeType.PROBABILITY_LIST);
		assertSame(3, tuple.size());
		assertEquals(0.3, tuple.get(0) , ATTRIBUTE_DELTA);
		assertEquals(0.2, tuple.get(1) , ATTRIBUTE_DELTA);
		assertEquals(0.1, tuple.get(2) , ATTRIBUTE_DELTA);
		
		//Endereignis
		node = nodes.get(0);
		assertTrue("Incorrect type", node instanceof CustomEvent);
		assertEquals("Endereignis", node.getName());
		assertEquals("_BJt6YNEaEeOs355zyZvogA", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		//und da ist die Kette zuende
		nodesTmp1 = new ArrayList<FlowNode>(epc.getDirectSuccessors(node));
		assertSame(0, nodesTmp1.size());
		
//		//oder Schleife:
		node = nodes.get(1);
		assertTrue("Incorrect type", node instanceof CustomEvent);
		assertEquals("E2", node.getName());
		assertEquals("_LbcZINErEeOrKdkTrDz40Q", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		
		nodesTmp2 = new ArrayList<FlowNode>(epc.getDirectSuccessors(nodes.get(1)));
		assertSame(1, nodesTmp2.size());
		node = nodesTmp2.get(0);
		assertTrue("Incorrect type", node instanceof CustomFunction);
		assertEquals("F2", node.getName());
		assertEquals("__su_ANEZEeOs355zyZvogA", ((IContainAttribute)node).getAttribute(AttributeType.BFLOW_ID));
		assertEquals(200.0, (Double) ((IContainAttribute)node).getAttribute(AttributeType.COST), ATTRIBUTE_DELTA);
		assertEquals(0.8, (Double) ((IContainAttribute)node).getAttribute(AttributeType.RELIABILITY), ATTRIBUTE_DELTA);
		assertEquals(20.0, (Double) ((IContainAttribute)node).getAttribute(AttributeType.TIME), ATTRIBUTE_DELTA);
	}
	
	//TODO: Testen: Schleifen
}
