package de.fh_zwickau.bumb_l_b.qos_tool.printer;

import static org.junit.Assert.*;

import java.io.ByteArrayOutputStream;
import java.io.PrintStream;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

public class BflowPrinterTest {
	private BflowPrinter printer = BflowPrinter.getInstance();
	private final ByteArrayOutputStream bos = new ByteArrayOutputStream();
	private final PrintStream ps = new PrintStream(bos);
	
	@Before
	public void setUp() throws Exception {
		bos.reset();
		printer.setOut(ps);
	}
	
	@Test
	public void testGetInstanceSingleton() {
		assertSame(printer, BflowPrinter.getInstance());
	}

	@Test
	public void testPrintError() {
		printer.printError("123 Test");
		assertEquals("addon:[MESSAGE][ERROR][][123 Test][]#FS#"+System.lineSeparator(), bos.toString());
	}

	@Test
	public void testPrintWarning() {
		printer.printWarning("123 Test");
		assertEquals("addon:[MESSAGE][WARNING][][123 Test][]#FS#"+System.lineSeparator(), bos.toString());
	}

	@Test
	public void testPrintInfo() {
		printer.printInfo("123 Test");
		assertEquals("addon:[MESSAGE][INFO][][123 Test][]#FS#"+System.lineSeparator(), bos.toString());
	}

	@Test
	public void testPrintErrorWithReference() {
		printer.printError("123 Test", "9Reference9");
		assertEquals("addon:[MESSAGE][ERROR][9Reference9][123 Test][]#FS#"+System.lineSeparator(), bos.toString());
	}

	@Test
	public void testPrintWarningWithReference() {
		printer.printWarning("123 Test", "9Reference9");
		assertEquals("addon:[MESSAGE][WARNING][9Reference9][123 Test][]#FS#"+System.lineSeparator(), bos.toString());
	}

	@Test
	public void testPrintInfoWithReference() {
		printer.printInfo("123 Test", "9Reference9");
		assertEquals("addon:[MESSAGE][INFO][9Reference9][123 Test][]#FS#"+System.lineSeparator(), bos.toString());
	}


}
