package de.fh_zwickau.bumb_l_b.qos_tool.printer;

import static org.junit.Assert.*;

import java.io.ByteArrayOutputStream;
import java.io.PrintStream;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

public class DefaultPrinterTest {
	DefaultPrinter printer = DefaultPrinter.getInstance();
	final ByteArrayOutputStream bos = new ByteArrayOutputStream();
	final PrintStream ps = new PrintStream(bos);
	
	@Before
	public void setUp() throws Exception {
		bos.reset();
		printer.setOut(ps);
	}
	
	@Test
	public void testGetInstanceSingleton() {
		assertSame(printer, DefaultPrinter.getInstance());
	}

	@Test
	public void testPrintError() {
		printer.printError("123 Test");
		assertEquals("ERROR: 123 Test", bos.toString().trim());
	}

	@Test
	public void testPrintWarning() {
		printer.printWarning("123 Test");
		assertEquals("WARNING: 123 Test", bos.toString().trim());
	}

	@Test
	public void testPrintInfo() {
		printer.printInfo("123 Test");
		assertEquals("INFO: 123 Test", bos.toString().trim());
	}

	@Test
	public void testPrintErrorWithReference() {
		printer.printError("123 Test", "9Reference9");
		assertEquals("ERROR: 123 Test (related to element 9Reference9)", bos.toString().trim());
	}

	@Test
	public void testPrintWarningWithReference() {
		printer.printWarning("123 Test", "9Reference9");
		assertEquals("WARNING: 123 Test (related to element 9Reference9)", bos.toString().trim());
	}

	@Test
	public void testPrintInfoWithReference() {
		printer.printInfo("123 Test", "9Reference9");
		assertEquals("INFO: 123 Test (related to element 9Reference9)", bos.toString().trim());
	}

}
