package de.fh_zwickau.bumb_l_b.qos_tool.processor;

import static org.junit.Assert.*;

import org.junit.Before;
import org.junit.Test;

public class CombinatorTest {
	Combinator c1;
	
	@Before
	public void setUp() throws Exception {
	}

	@Test (expected = IllegalArgumentException.class)
	public void testInvalidCombinator1() {
		new Combinator(0, 0);
	}

	@Test (expected = IllegalArgumentException.class)
	public void testInvalidCombinator2() {
		new Combinator(1, 0);
	}
	
	@Test (expected = IllegalArgumentException.class)
	public void testInvalidCombinator3() {
		new Combinator(1, 2);
	}
	
	@Test (expected = IllegalArgumentException.class)
	public void testInvalidCombinator4() {
		new Combinator(-1, -1);
	}
	
	@Test
	public void testReset() {
		c1 = new Combinator(5, 3);
		
		assertArrayEquals(new int[] {0,1,2}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {0,1,3}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {0,1,4}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {0,2,3}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {0,2,4}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {0,3,4}, c1.getCombination()); c1.next();
		
		c1.reset();
		
		assertArrayEquals(new int[] {0,1,2}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {0,1,3}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {0,1,4}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {0,2,3}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {0,2,4}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {0,3,4}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {1,2,3}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {1,2,4}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {1,3,4}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {2,3,4}, c1.getCombination());
	}

	@Test
	public void testNext() {
		c1 = new Combinator(5, 3);
		assertArrayEquals(new int[] {0,1,2}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {0,1,3}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {0,1,4}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {0,2,3}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {0,2,4}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {0,3,4}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {1,2,3}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {1,2,4}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {1,3,4}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {2,3,4}, c1.getCombination());
		c1.next();
		c1.next();
		c1.next();
		assertArrayEquals("have to keep last element if reached", new int[] {2,3,4}, c1.getCombination());
	}

	@Test
	public void testGetCombination1() {
		c1 = new Combinator(1, 1);
		assertArrayEquals(new int[] {0}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {0}, c1.getCombination()); c1.next();
	}

	@Test
	public void testGetCombination2() {
		c1 = new Combinator(2, 1);
		assertArrayEquals(new int[] {0}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {1}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {1}, c1.getCombination()); c1.next();
	}
	
	@Test
	public void testGetCombination3() {
		c1 = new Combinator(2, 2);
		assertArrayEquals(new int[] {0, 1}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {0, 1}, c1.getCombination()); c1.next();
	}
	
	@Test
	public void testGetCombination4() {
		c1 = new Combinator(3, 1);
		assertArrayEquals(new int[] {0}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {1}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {2}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {2}, c1.getCombination()); c1.next();
	}
	
	@Test
	public void testGetCombination5() {
		c1 = new Combinator(3, 2);
		assertArrayEquals(new int[] {0,1}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {0,2}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {1,2}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {1,2}, c1.getCombination()); c1.next();
	}
	
	@Test
	public void testGetCombination6() {
		c1 = new Combinator(3, 3);
		assertArrayEquals(new int[] {0,1,2}, c1.getCombination()); c1.next();
		assertArrayEquals(new int[] {0,1,2}, c1.getCombination()); c1.next();
	}
	
	@Test
	public void testGetCombinationCount() {
		c1 = new Combinator(1, 1);
		assertEquals(1, c1.getCombinationCount());
		c1 = new Combinator(2, 1);
		assertEquals(2, c1.getCombinationCount());
		c1 = new Combinator(2, 2);
		assertEquals(1, c1.getCombinationCount());
		c1 = new Combinator(10, 4);
		assertEquals(210, c1.getCombinationCount());
		c1 = new Combinator(10, 10);
		assertEquals(1, c1.getCombinationCount());
	}

	@Test (expected = ArithmeticException.class)
	public void testGetCombinationCountOverflow() {
		c1 = new Combinator(1000, 500);
	}
	
}
